# packages/survey/lib/respond.tcl
#
# includelet for responding to a survey section.  this will take care
# of collating the different questions if the sections are located on
# a different section.  the granularity as of now is up to the section level.
#
# @author Deds Castillo (deds@i-manila.com.ph)
# @creation-date 2004-07-07
# @arch-tag: 75d019e9-22a8-4db8-b7ec-4261aeab9360
# @cvs-id $Id: respond-new.tcl,v 1.2 2005/01/13 13:54:39 jeffd Exp $

#foreach required_param {survey_id category_id} {
#    if {![info exists $required_param]} {
#        return -code error "$required_param is a required parameter."
#    }
#}

foreach required_param {section_ids} {
    if {![info exists $required_param]} {
        return -code error "$required_param is a required parameter."
    }
}

ns_log Notice "DEDS:[llength $section_ids]"

if {![info exists response_id]} {
    set response_id [db_nextval acs_object_id_seq]
}
set return_url "../"

set has_edit 0
set mode edit

set form_name "survey_form"

ad_form \
    -name $form_name \
    -html {enctype multipart/form-data} \
    -has_edit $has_edit \
    -cancel_url $return_url \
    -form {
        {section_ids:text(hidden),multiple {values $section_ids}}
        {response_id:text(hidden) {value $response_id}}
        {return_url:text(hidden) {value $return_url}}
    }

foreach section_id $section_ids {
    survey::form::add_survey_section \
        -form $form_name \
        -section_id $section_id \
        -response_id $response_id
}

ad_form \
    -extend  \
    -name $form_name \
    -after_submit {
        ns_log Notice "we should not be here yet"
#        rp_internal_redirect "/packages/survey/www/process-response"
#        ad_script_abort
        ad_returnredirect $return_url
    }

set style "survey/standard"

