# packages/survey/lib/respond.tcl
#
# includelet for responding to a survey
#
# @author Deds Castillo (deds@i-manila.com.ph)
# @creation-date 2004-07-07
# @arch-tag: 75d019e9-22a8-4db8-b7ec-4261aeab9360
# @cvs-id $Id: respond.tcl,v 1.2 2005/01/13 13:54:39 jeffd Exp $

foreach required_param {survey_id category_id} {
    if {![info exists $required_param]} {
        return -code error "$required_param is a required parameter."
    }
}
foreach optional_param {form_name section_ids response_id return_url edit_p mode element_id action_url} {
    if {![info exists $optional_param]} {
        set $optional_param {}
    }
}

if {!$response_id} {
    set response_id [db_nextval acs_object_id_seq]
    set section_ids [survey_next_sections $survey_id 0 $response_id]
    set edit_label "Add"
} else {
    set section_ids [survey_next_sections -edit_p t $survey_id 0 $response_id]
    set edit_label "Edit"
}

if {[empty_string_p $return_url]} {
    set return_url [ad_return_url]
}

set has_edit 0

ns_log Notice "DEDS:$survey_id:$mode"

#set has_edit [expr ![string equal $mode edit]] 
set has_edit 0

if {[empty_string_p $form_name]} {
    set form_name "survey_form_$survey_id"
}

if {[empty_string_p $action_url]} {
    ad_form \
        -name $form_name \
        -html {enctype multipart/form-data} \
        -has_edit $has_edit \
        -cancel_url $return_url \
        -mode $mode \
        -form {
            {survey_id:text(hidden) {value $survey_id}}
            {edit_element_id:text(hidden) {value $element_id}}
            {section_ids:text(hidden) {value $section_ids}}
            {response_id:text(hidden) {value $response_id}}
            {return_url:text(hidden) {value $return_url}}
            {element_id:text(hidden) {value $element_id}}
            {category_id:text(hidden) {value $category_id}}
        }
} else {
    ad_form \
        -name $form_name \
        -html {enctype multipart/form-data} \
        -action $action_url \
        -has_edit $has_edit \
        -cancel_url $return_url \
        -mode $mode \
        -form {
            {survey_id:text(hidden) {value $survey_id}}
            {edit_element_id:text(hidden) {value $element_id}}
            {section_ids:text(hidden) {value $section_ids}}
            {response_id:text(hidden) {value $response_id}}
            {return_url:text(hidden) {value $return_url}}
            {element_id:text(hidden) {value $element_id}}
            {category_id:text(hidden) {value $category_id}}
        }
}

foreach section_id $section_ids {
    survey::form::add_survey_section \
        -form $form_name \
        -section_id $section_id \
        -response_id $response_id \
        -mode $mode \
	-edit_p $edit_p
}

ad_form \
    -extend  \
    -name $form_name \
    -on_request {
        foreach section_id $section_ids {
            survey::form::set_value_survey_section -form take_survey -section_id $section_id -response_id $response_id
        }
    } \
    -after_submit {
        rp_internal_redirect "/packages/survey/www/process-response"
        ad_script_abort
    }

set style "survey/standard"

db_0or1row get_style {
    select 'survey/'||template_file as style 
    from survey_templates t, 
         surveys s
    where s.survey_id = :survey_id
          and s.template = t.template_id
}
