# packages/survey/tcl/survey-form-procs.tcl

ad_library {
    
    procs for adding survey to form
    
    @author Deds Castillo (deds@i-manila.com.ph)
    @creation-date 2004-07-07
    @arch-tag: 9987c365-8790-43c8-bbb5-59d1d223b852
    @cvs-id $Id: survey-form-procs.tcl,v 1.4 2005/03/23 22:23:40 jeffd Exp $
}

namespace eval survey::form {}

ad_proc -public survey::form::add_survey_section {
    {-form:required}
    {-section_id:required}
    {-response_id 0}
    {-mode edit}
    {-edit_p f}
} {
    adds a whole section to a form
    
    @author Deds Castillo (deds@i-manila.com.ph)
    @creation-date 2004-07-07
    
    @param form
    
    @param section_id
    
    @param response_id
    
    @return 
    
    @error 
} {
    
    get_section_info -section_id $section_id

    # create the section
    ad_form \
        -extend \
        -name $form \
        -form [list \
                   [list "section_info$section_id:text(inform)" {label ""} {value $section_info(description)} {section $section_info(name_unquoted)}]
              ]

    db_foreach get_questions {} {
        survey::form::create_ad_form_element \
            -form $form \
            -question_id $question_id \
            -response_id $response_id \
            -mode $mode \
	    -edit_p $edit_p
    }
}
        
        
ad_proc -public survey::form::create_ad_form_element {
    {-form:required}
    {-question_id:required}
    {-response_id 0}
    {-mode edit}
    {-edit_p f}
} {
    creates a form element
    
    @author Deds Castillo (deds@i-manila.com.ph)
    @creation-date 2004-07-07
    
    @param form
    
    @param question_id
    
    @param response_id
    
    @return 
    
    @error 
} {
    
    if {[string equal $edit_p t]} {
        set edit_previous_response_p 1
    } else {
        set edit_previous_response_p 0
    }

    set element_name "response_to_question.$question_id"

    db_1row get_question_properties {}

    set user_value ""
    set selected_choices [list]

    if { $predefined_question_id != [db_null] } {
        # question is a predefined question. If it has already been answered, get the last response value from the db
        # if it is a predefined question with action_type="db" get the value from the db instead.

        db_1row predefined_question_data {}
        
        if {[string equal $action_type "db"]} {
            # select preselected value from the db
            set user_id [ad_conn user_id]
            
            # We have to check for date
            if {[string equal $abstract_data_type "date"]} {
                set user_value [db_string get_value "
                    select to_char(:column_name,'YYYY MM DD')
                    from $table_name where $key_name = :user_id
                " -default ""]
                set date_answer $user_value
            } else {
                set user_value [db_string get_value "
                    select $column_name 
                    from $table_name 
                    where $key_name = :user_id
                " -default ""]
                set choice_id $user_value
                set boolean_answer $user_value
                set clob_answer $user_value
                set number_answer $user_value
                set varchar_answer $user_value
                set attachment_answer $user_value
            }
        } else {
            # select preselected value from latest response
            set user_id [ad_conn user_id]
            if {[db_0or1row locate_predef_preselect {}]} {
                set count 0
                db_foreach prev_response_query_predef {} {
                    incr count
		    if {![empty_string_p $choice_id]} {
			lappend selected_choices $choice_id
		    }
                } if_no_rows {
                    set choice_id 0
                    set boolean_answer ""
                    set clob_answer ""
                    set number_answer ""
                    set varchar_answer ""
                    set date_answer ""
                    set attachment_answer ""
                }
                set edit_previous_response_p 1
            } else {    
                set choice_id 0
                set boolean_answer ""
                set clob_answer ""
                set number_answer ""
                set varchar_answer ""
                set date_answer ""
                set attachment_answer ""
            }
        }
    } else {
        if {$edit_previous_response_p} {
            set user_id [ad_conn user_id]
	    
            set count 0
            db_foreach prev_response_query {} {
                incr count
                if {![empty_string_p $choice_id]} {
		    lappend selected_choices $choice_id
                }
            } if_no_rows {
                set choice_id 0
                set boolean_answer ""
                set clob_answer ""
                set number_answer ""
                set varchar_answer ""
                set date_answer ""
                set attachment_answer ""
            }
        }
    }

    if {$edit_previous_response_p} {
        switch -- $abstract_data_type {
            "choice" {
		set user_value [join $selected_choices " "]
            }
            "shorttext" {
                set user_value $varchar_answer
            }
            "boolean" {
                set user_value $boolean_answer
            }
            "integer" -
            "number" {
                set user_value $number_answer
            }
            "text" {
                set user_value $clob_answer
            }
            "date" {
                set user_value $date_answer
            }
            "blob" {
                if {![empty_string_p $attachment_answer]} {
                    set package_id [ad_conn package_id]
                    set filename [db_string get_file_name {
                        select name 
                        from cr_items i, 
                             cr_revisions r 
                        where i.item_id=r.item_id 
                              and r.revision_id = :attachment_answer
                    }]
                    regsub "_$response_id$" $filename "" filename		   
                    set user_value "Uploaded file: <a href=\"[site_node::get_url_from_object_id -object_id $package_id]/view-attachment?[export_url_vars response_id question_id]\">\"$filename\"</a>"
                }
            }
        }
    }

    set pretty_user_value $user_value

    # add the form elements, depending on the presentation type
    switch -- $presentation_type {
        "upload_file"  {
            set element_type "file(file)"
        }
        "textbox" {
            set element_type "text(text)"
            set html [ad_decode $presentation_options "large" {size 70} "medium" {size 40} {size 10}]
            set element_param_list {{html $html}}
        }
        "textarea" {
            set element_type "text(textarea)"
            set html [ad_decode $presentation_options "large" {rows 20 cols 65} "medium" {rows 15 cols 55} {rows 8 cols 35}] 
            set element_param_list {{html $html}}
	    }
        "date" {
            set element_type "date(date)"
        }
	"select_text" -
        "select" {
            if { [string equal $abstract_data_type "boolean"] } {
                if {![empty_string_p $presentation_options]} {
                    set options_list [split $presentation_options "/"]
                    set choice_t [lindex $options_list 0]
                    set choice_f [lindex $options_list 1]
                } else {
                    set choice_t "True"
                    set choice_f "False"
                }
                set options "{$choice_t t} {$choice_f f}"
                set pretty_user_value [string map {t $choice_t f $choice_f} $user_value]
            } else {
                # at some point, we may want to add a UI option for the admin
                # to sepcify multiple or not for select
                set optionlist [list]
                db_foreach question_choices {} {
                    lappend optionlist [list $label $choice_id]
                }
                set options $optionlist
                set pretty_user_value [db_string get_pretty_value {} -default ""]
            }
	    if { $presentation_type == "select_text" } {
		set element_type "text(select_text)"
	    } else {
		set element_type "text(select)"
	    }
            set element_param_list {{options $options}}
        }
	"radio_text" -
        "radio" {
            if { [string equal $abstract_data_type "boolean"] } {
                if {![empty_string_p $presentation_options]} {
                    set options_list [split $presentation_options "/"]
                    set choice_t [lindex $options_list 0]
                    set choice_f [lindex $options_list 1]
                } else {
                    set choice_t "True"
                    set choice_f "False"
                }
                
                set options "{$choice_t t} {$choice_f f}"
                set pretty_user_value [string map {t $choice_t f $choice_f} $user_value]
            } else {
                set optionlist [list]
                db_foreach question_choices {} {
                    # replace {xxx.gif} by image inclusion
                    regsub -all "\{(\[^\{\}\]*)\}" $label {<img src=/graphics/survey/\1>} label
                    lappend optionlist [list $label $choice_id]
                }
                set options $optionlist
                set pretty_user_value [db_string get_pretty_value {} -default ""]
            }  
	    if {$presentation_type == "radio_text"} {
		set element_type "text(radio_text)"
	    } else {
		set element_type "text(radio)"
	    }
            set element_param_list {{options $options}}
        }
	"checkbox_text" -
        "checkbox" {
            set choices [list]
            set optionlist [list]
            db_foreach question_choices_all {} {
                lappend optionlist [list $label $choice_id]
            }
            set options $optionlist
	    if { $presentation_type == "checkbox_text" } {
		set element_type "text(checkbox_text)"
	    } else {
		set element_type "text(checkbox)"
	    }
            set element_param_list {{options $options}}
        }
    }

    if {$edit_previous_response_p} {
        # we are editing so set a value
        if {[string equal $presentation_type checkbox]} {
            # we allow multiple values here only if not on display
            # mode
            if {[string equal $mode display]} {
                set pretty_user_value_list [db_list get_choice_label {}]
                set pretty_user_value [join $pretty_user_value_list ", "]
                regsub -all -- {\$} $pretty_user_value {\\$} pretty_user_value
                lappend element_param_list [list value $pretty_user_value]
            } else {
                lappend element_param_list [list values $user_value]
            }
        } elseif { $presentation_type == "checkbox_text" } {
	    # SB: checkbox_text widget expects "value" to be in the format {{choice ids} text}
	    set text_value [survey::form::get_response_text -response_id $response_id -question_id $question_id \
				-abstract_data_type "choice"]
            if {[string equal $mode display]} {
                set pretty_user_value_list [db_list get_choice_label {}]
                set pretty_user_value [join $pretty_user_value_list ", "]
                regsub -all -- {\$} $pretty_user_value {\\$} pretty_user_value
                lappend element_param_list [list value $pretty_user_value]
            } else {
                lappend element_param_list [list value "\{$user_value\} \"$text_value\""]
            }
	} elseif { $presentation_type == "select_text" || $presentation_type == "radio_text" } { 
	    set text_value [survey::form::get_response_text -response_id $response_id -question_id $question_id \
				-abstract_data_type $abstract_data_type]
            if {[string equal $mode display]} {
                if { ![string equal $abstract_data_type "boolean"] } {
                    # boolean is a special case so we do not replace
                    # their dollar signs
                    regsub -all -- {\$} $pretty_user_value {\\$} pretty_user_value
                }
                # this takes care of textareas
                regsub -all -- {\n} $pretty_user_value {<br />} pretty_user_value
                lappend element_param_list [list value $pretty_user_value]
            } else {
                lappend element_param_list [list value "$user_value \"$text_value\""]
            }
	} else {
            if {[string equal $mode display]} {
                if { ![string equal $abstract_data_type "boolean"] } {
                    # boolean is a special case so we do not replace
                    # their dollar signs
                    regsub -all -- {\$} $pretty_user_value {\\$} pretty_user_value
                }
                # this takes care of textareas
                regsub -all -- {\n} $pretty_user_value {<br />} pretty_user_value
                lappend element_param_list [list value $pretty_user_value]
            } else {
                lappend element_param_list [list value $user_value]
            }
        }
    }
    set optional ""
    if {!$required_p} {
        set optional ",optional"
    }

    if {[string equal $mode display]} {
        if {[string equal $edit_p t]} {
            set element_type "text(inform)"
        } else {
            set element_type "text(hidden)"
        }
    }

    lappend element_param_list [list label $question_text]

    set element_params [concat [list "${element_name}:${element_type}${optional}"] $element_param_list]

    ns_log Notice "\nDEDS\nediting? $edit_previous_response_p\ncreating element using -- $element_params"

    ad_form \
        -extend \
        -name $form \
        -form [list $element_params]
    
}

ad_proc -public survey::form::get_response_text {
    {-response_id:required}
    {-question_id:required}
    {-abstract_data_type:required}
} {
    Utility procedure for checkbox_text, radio_text and select_text widgets.
    Returns the value of the text part of the widget. 
    Used for retrieving the response for editing purposes.
    
    @author simon@simonbuckle.com
} {
    set txt ""

    # only return the text if the widget is one of the allowed types
    switch -- $abstract_data_type {
	"choice" {
	    set txt [db_string choice_text_value {} -default ""]
	}
	"boolean" {
	    set txt [db_string boolean_text_value {} -default ""]
	}
    }

    return $txt
}