ad_library {
    Search service contract implementations

    @author Dave Bauer (dave@thedesignexperience.org)
    @creation-date 2004-09-17
    @arch-tag: 05727441-9392-4d97-923b-bc5cf53a0b3d
    @cvs-id $Id: survey-search-procs.tcl,v 1.2 2005/03/26 21:55:35 jeffd Exp $
}

namespace eval ::survey::search {}
namespace eval ::survey::search::survey {}
namespace eval ::survey::search::section {}
namespace eval ::survey::search::question {}

ad_proc -private ::survey::search::survey::datasource {
    object_id
} {
    Provide datasource for FtsContentProvider service contract

    @author Dave Bauer (dave@thedesignexperience.org)
    @creation-date 2004-09-17

    @param object_id

    @return

    @error
} {
    set url [survey::search::survey::url $object_id]

    db_1row get_survey {
        select o.object_id,
          s.name as title,
          s.description,
          o.last_modified as pubdate,
          'text/plain' as mime,
          'text' as storage_type,
          '' as keywords
        from surveys s, acs_objects o
        where s.survey_id=:object_id
          and s.survey_id=o.object_id
    }
    
    return [list object_id $object_id \
                title $title \
                description $description \
                content "$title $description" \
                pubdate $pubdate \
                mime text/plain \
                storage_type text \
                keywords {} \
                syndication \
                [list \
                     link "[ad_url]$url" \
                     description $title \
                     author $email \
                     category Survey \
                     guid "[ad_url]/o/$object_id" \
                     pubDate $pubdate \
                    ] \
               ]
}

ad_proc -private ::survey::search::survey::url {
    object_id
} {
    Generate URL for search service contract

    @author Dave Bauer (dave@thedesignexperience.org)
    @creation-date 2004-09-17

    @param object_id

    @return

    @error
} {
    set package_id [db_string get_package_id "select package_id from surveys where survey_id=:object_id"]

    array set sn [site_node::get_from_object_id -object_id $package_id]

    return "[export_vars -base $sn(url)one-survey {{survey_id $object_id}}]"
}


ad_proc -private ::survey::search::section::datasource {
    object_id
} {
    Provide datasource for FtsContentProvider service contract

    @author Dave Bauer (dave@thedesignexperience.org)
    @creation-date 2004-09-17

    @param object_id

    @return

    @error
} {

    db_1row get_survey_section {
        select o.object_id,
          s.name as title,
          s.description,
          o.last_modified as pubdate,
          'text/plain' as mime,
          'text' as storage_type,
          '' as keywords,
          (select email from acs_users_all where user_id = o.creation_user) as email
        from survey_sections s, acs_objects o
        where s.section_id=:object_id
          and s.section_id=o.object_id} -column_array datasource

    return [array get datasource]
}

ad_proc -private ::survey::search::section::url {
    object_id
} {
    Generate URL for search service contract

    @author Dave Bauer (dave@thedesignexperience.org)
    @creation-date 2004-09-17

    @param object_id is a section_id

    @return

    @error
} {
    db_1row get_package_survey_ids {
        select s.package_id, s.survey_id
        from survey_sections ss, surveys s
        where s.survey_id=ss.survey_id
          and ss.section_id=:object_id
    }
    array set sn [site_node::get_from_object_id -object_id $package_id]
    return "[export_vars -base $sn(url)admin/one {survey_id}]\#$object_id"
}


ad_proc -private ::survey::search::question::datasource {
    object_id
} {
    Provide datasource for FtsContentProvider service contract

    @author Dave Bauer (dave@thedesignexperience.org)
    @creation-date 2004-09-17

    @param object_id is a question_id

    @return
} {

    db_1row get_survey_question {select o.object_id,
                        sq.question_text as title,
                        sq.question_text as description,
                        sq.question_text as content,
                        o.last_modified as pubdate,
                        'text/plain' as mime,
                        'text' as storage_type,
                        '' as keywords
                        from survey_questions sq, acs_objects o
                        where sq.question_id=:object_id
                        and sq.question_id=o.object_id} -column_array datasource

    return [array get datasource]
}

ad_proc -private ::survey::search::question::url {
    object_id
} {
    Generate URL for search service contract

    @author Dave Bauer (dave@thedesignexperience.org)
    @creation-date 2004-09-17

    @param object_id

    @return

    @error
} {
    db_1row get_package_and_survey_ids {
        select s.survey_id, s.package_id
        from survey_questions sq, survey_sections ss, surveys s
        where s.survey_id=ss.survey_id
          and ss.section_id=sq.section_id
        and sq.question_id=:object_id}

    array set sn [site_node::get_from_object_id -object_id $package_id]

    return "[export_vars -base $sn(url)admin/one {survey_id}]\#$object_id"
}
