ad_page_contract {

    This page allows the admin to bulk upload responses to a survey
    
    @param  survey_id integer denoting survey we're administering
    @param  csvfile uploaded csv-file, whose fields will be
    interpreted as replies to the survey

    @author koljalehmann@uni.de
    @date   September 3rd, 2003
    @cvs-id $Id: import-responses.tcl,v 1.3 2005/03/16 12:26:31 jeffd Exp $
} {
    survey_id:integer
    {csvfile:array,optional}
    {filename ""}
} -validate {
    file_given {
	if {![info exists csvfile(tmpfile)] && $filename==""} {
	    ad_complain "[_ survey.lt_No_file_given_for_imp]"
	}
    }
}

set package_id [ad_conn package_id]

ad_require_permission $package_id admin

# Get the survey information.
get_survey_info -survey_id $survey_id
if {![info exists survey_info(survey_id)]} {
    ad_return_complaint 1 "[_ survey.lt_Requested_survey_does]"
    ad_script_abort
}
set survey_name $survey_info(name)
set context [list [list "one?[export_url_vars survey_id]" $survey_info(name)] [_ survey.Import_responses]]

if {[info exists csvfile(tmpfile)]} {
    # preserve the file for future use
    set filename [ns_tmpnam]
    file rename $csvfile(tmpfile) $filename
}

# First, create a list of all fields in the csv
set file [open $filename r]
set numfields [ns_getcsv $file fields]

set fieldlist [list [list "[_ survey.--_Please_Choose_--]" ""]]
set i 0
foreach field $fields {
    lappend fieldlist [list $field $i]
    incr i
}

ad_form -name pick-fields -form {
    {survey_id:text(hidden) {value $survey_id}}
    {filename:text(hidden) {value $filename}}
    {identifier:text(select) {label "[_ survey.Identify_user_by]"} {options {{UserID user_id} {Email email}}}}
    {idfield:text(select) {label "[_ survey.Identifying_field]"} {options $fieldlist} {required_p "t"}}
}

set question_ids [list]
set section ""
db_foreach get_questions {} {
    if {$section_desc != $section} {
	template::form::section pick-fields $section_desc
	set section $section_desc
    }
    template::element create pick-fields question$question_id \
	-widget select \
	-label $question_text \
	-options $fieldlist \
	-required_p $required_p
    lappend question_ids $question_id
    set question($question_id.datatype) $abstract_data_type
    set question($question_id.required_p) $required_p
    set question($question_id.question_text) $question_text
    if {$abstract_data_type=="choice"} {
	# get the possible choices
	db_foreach get_choices "" {
	    set question($question_id.choices.$label) $choice_id
	}
    }
}

ad_form -extend -name pick-fields -form {
    {test:text(submit) {label "[_ survey.Test]"} {value "test"}}
    {import:text(submit) {label "[_ survey.Import]" {value "import"}}
} -on_submit {
    # Now, get the data ready to insert into the db
    if {[info exists import] && $import=="import"} {
	set mode "import"
    }
    if {[info exists test] && $test=="test"} {
	set mode "test"
    }

    # First, get the user id, the response belongs to
    set debug ""
    set errors ""
    set line 0
    while {[ns_getcsv $file data]!=-1} {
	set error 0
	incr line
	db_transaction {
	    if {$identifier=="user_id"} {
		set user_id [lindex $data $idfield]
		if {[db_0or1row user_exists ""]==0} {
		    # no user for this user_id
		    set error 1
		}
	    } else {
		set email [lindex $data $idfield]
		if {![db_0or1row get_user_id ""]} {
		    # no user for this email
		    set error 1
		}
	    }
	    if {$error} {
		# user does not exist
		lappend errors "[_ survey.lt_Line_line_User_with_i]"
		db_abort_transaction
	    }
	    append debug "$user_id:"
	
	    # Now, get a response_id
	    # If it is a "single_response" survey, overwrite if necessary
ns_log notice 1
	    if {[db_0or1row get_response_id ""]} {
		set overwrite 1
		# delete old response
		db_dml delete_old_response ""
	    } else {
		# No response_id to overwrite
		set response_id [db_nextval acs_object_id_seq]
		set overwrite 0
		# create the response
		set creation_ip [ad_conn peeraddr]
		set initial_response_id $response_id
		db_exec_plsql create_response {}
	    }
ns_log notice 2

	    # Now, get the question responses ready
	    foreach question_id $question_ids {
		set response_value [lindex $data [set question$question_id]]
		if {[empty_string_p $response_value]} {
		    # there is an empty value
		    # If question is not required, no problem, just skip it
		    # else, note an error
		    if {$question($question_id.required_p)=="t"} {
			set error 1
			lappend errors "[_ survey.lt_Line_line_Answer_to_q]"
		    }
		    continue
		}
		switch $question($question_id.datatype) {
		    choice {
			foreach choice [split $response_value ","] {
			    if {[info exists question($question_id.choices.$choice)]} {
				set choice_id $question($question_id.choices.$choice)
				db_dml create_choice_response ""
			    } else {
				set error 1
				lappend errors "[_ survey.lt_Line_line_Choice_choi]"
			    }
			}
		    }
		    shorttext {
			set varchar_answer $response_value
			db_dml create_varchar_response ""
		    }
		    boolean {
			switch $response_value {
			    "Yes" -
			    "True" {
				set boolean_answer "t"
				db_dml create_boolean_response ""
			    }
			    "No" -
			    "False" {
                                set boolean_answer "f"
                                db_dml create_boolean_response ""
                            }
			    default {
				set error 1
				lappend errors "[_ survey.lt_Line_line_Invalid_boo]"
			    }
			}
		    }
		    integer -
		    number {
			if {[ad_var_type_check_number_p $response_value]} {
			    set number_answer $response_value
			    db_dml create_number_response ""
			} else {
			    set error 1
			    lappend errors "[_ survey.lt_Line_line_Invalid_Num]"
			}
		    }
		    text {
			db_dml create_clob_response {} -clobs [list $response_value]
		    }
		    date {
			if [regexp {(.*)-(.*)-(.*)$} $response_value match year month day] {
			    set date_answer $response_value
			    db_dml create_date_response ""
                        } else {
                            set error 1
                            lappend errors "[_ survey.lt_Line_line_Invalid_Dat]"
			}
		    }
		    blob {
			# Just do nothing
		    }
		}
		
	    }
	    if {$error||$mode=="test"} {
		db_abort_transaction
	    }
	} on_error {
	    # There needs to be an on_error block
	    if [exists_and_not_null errmsg] {
		ns_returnnotice 200 "[_ survey.errmsg__line]"
	    }
	}

    }
    if {[llength $errors]==0} {
	# No errors
	if {$mode=="test"} {
	    ns_returnnotice 200 "[_ survey.lt_Test_successful___The]"
	} else {
	    ns_returnnotice 200 "[_ survey.lt_Import_successful___T]"
	}
    } else {
	# There were errors
	ns_returnnotice 200 "[_ survey.Errors_encountered]" ""[_ survey.lt_There_were_some_error]": <ul><li>[join $errors "<li>"]</ul>"
    }
}
close $file

