# /www/survsimp/admin/question-add-3.tcl
ad_page_contract {
    Inserts a new question into the database based on a predefined question.

    @param question_id             id of new question
    @param section_id          integer determining survey we're dealing with
    @param after                   optional integer determining position of this question
    @param predefined_question_id  id of the predefined question to copy
    @param required_p              flag telling us whether an answer to this question is mandatory
    @param active_p                flag telling us whether this question will show up at all

    @author Kolja Lehmann
    @cvs-id $Id: predefined-question-2.tcl,v 1.3 2004/02/14 19:57:59 maltes Exp $
} {
    predefined_question_id:integer,notnull
    section_id:integer,notnull
    question_id:integer,notnull
    after:integer,optional
    {required_p t}
    {active_p t}
    {question_text ""}
}

# Variables that will be used later...
set question_html_p "f"
set summary_type [db_null]
set answer_description [db_null]
###

set package_id [ad_conn package_id]
set user_id [ad_get_user_id]
ad_require_permission $package_id admin
get_survey_info -section_id $section_id
set survey_id $survey_info(survey_id)
get_section_info -section_id $section_id

if {![info exists survey_info(survey_id)]} {
    ad_return_complaint 1 "[_ survey.lt_Requested_survey_does]"
    ad_script_abort
}

set survey_id $survey_info(survey_id)
set exception_count 0
set exception_text ""

if { $exception_count > 0 } {
    ad_return_complaint $exception_count $exception_text
    ad_script_abort
}

set already_inserted_p [db_string already_inserted_p {}]

if { $already_inserted_p } {
    ad_returnredirect "one?[export_vars survey_id]"
    ad_script_abort
}

   set new_question_text $question_text
   db_1row get_predefinition ""
   if {![empty_string_p $new_question_text]} {
       set question_text $new_question_text
   }

   db_transaction {
	if { [exists_and_not_null after] } {
	    # We're inserting between existing questions; move everybody down.
	    set sort_order [expr { $after + 1 }]
	    db_dml renumber_sort_orders {}
	} else {
	    set sort_order [expr [db_string max_question {}] + 1]
	}

       db_exec_plsql create_question {}
#       ns_returnnotice 200 test
       #db_0or1row prepare_add_question_text {}
       db_dml add_question_text {}


	# For questions where the user is selecting a canned response, insert
	# the canned responses into survey_question_choices by using the predefined
	# choices

	if { $presentation_type == "checkbox" || $presentation_type == "radio" || $presentation_type == "select" } {
	    if { $abstract_data_type == "choice" } {
		db_dml add_new_choices ""
            }
	}
} on_error {
            db_release_unused_handles
            ad_return_error "Database Error" "<pre>$errmsg</pre>"
            ad_script_abort

}


db_release_unused_handles
ad_returnredirect "one?survey_id=$survey_id&#${sort_order}"







