ad_page_contract {

    Insert user response into database.
    This page receives an input for each question named
    response_to_question.$question_id 

    @param   survey_id             survey user is responding to
    @param   section_ids           sections on the last page
    @param   return_url            optional redirect address

    @param   response_to_question  since form variables are now named as response_to_question.$question_id, this is actually array holding user responses to all survey questions.
    
    @author  jsc@arsdigita.com
    @author  nstrug@arsdigita.com
    @date    28th September 2000
    @cvs-id $Id: process-response.tcl,v 1.2 2005/01/13 13:54:40 jeffd Exp $
} {
    survey_id:integer
    section_ids
    {initial_response_id:integer 0}
    {return_url:optional ""}
    response_to_question:array,optional,multiple,html
    response_id:integer,notnull
    {edit_p "f"}
} -validate {
    sections_exist -requires { section_ids } {
	foreach section_id $section_ids {
	    if {[ad_var_type_check_integer_p $section_id]} {
		if ![db_0or1row section_exists {}] {
		    ad_complain "[_ survey.lt_Section_section_id_do_1]"
		}
	    } else {
		ad_complain "[_ survey.lt_Section_id_section_id]"
	    }
	}
    }

    check_questions -requires { section_ids } {
	set question_info_list [list]
	foreach section_id $section_ids {
	    set question_info_list [concat $question_info_list [db_list_of_lists survey_question_info_list {}]]
	}
	## Validate input.
	
	set questions_with_missing_responses [list]

	foreach question $question_info_list { 
	    set question_id [lindex $question 0]
	    set question_text [lindex $question 1]
	    set abstract_data_type [lindex $question 2]
	    set required_p [lindex $question 4]

	    #  Need to clean-up after mess with :array,multiple flags
	    #  in ad_page_contract.  Because :multiple flag will sorround empty
	    #  strings and all multiword values with one level of curly braces {}
	    #  we need to get rid of them for almost any abstract_data_type
	    #  except 'choice', where this is intended behaviour.  Why bother
	    #  with :multiple flag at all?  Because otherwise we would lost all
	    #  but first value for 'choice' abstract_data_type - see ad_page_contract
	    #  doc and code for more info.
	    #
	    if { [exists_and_not_null response_to_question($question_id)] } {
		if {$abstract_data_type != "choice"} {
		    set response_to_question($question_id) [join $response_to_question($question_id)]
		} else {
		    if { [empty_string_p [lindex $response_to_question($question_id) 0 ] ] } {
			set response_to_question($question_id) ""
		    }
		}
	    }
	    
	    if { $abstract_data_type == "date" } {
		if [catch  { set response_to_question($question_id) [validate_ad_dateentrywidget "" response_to_question.$question_id [ns_getform]]} errmsg] {
		    ad_complain "[_ survey.lt_errmsg_Please_make_su]"
		}
	    }
	    
	    if {$abstract_data_type=="blob"} {
		# check if file was already uploaded earlier
		if {[db_0or1row upload_file_exist ""]} {
		    set upload_file_exist($question_id) $attachment_answer
		}
	    }
	    
	    if { [exists_and_not_null response_to_question($question_id)] } {
		
		set response_value [string trim $response_to_question($question_id)]
	    } elseif {$required_p == "t" && ![info exists upload_file_exist($question_id)]} {
		lappend questions_with_missing_responses $question_text
		continue
	    } else {
		set response_to_question($question_id) ""
		set response_value ""
	    }
	    
	    if {![empty_string_p $response_value]} {
		if { $abstract_data_type == "number" } {
		    if { ![regexp {^(-?[0-9]+\.)?[0-9]+$} $response_value] } {
			
			ad_complain "[_ survey.lt_The_response_to_quest]"
			continue
		    }
		} elseif { $abstract_data_type == "integer" } {
		    if { ![regexp {^[0-9]+$} $response_value] } {
			
			ad_complain "[_ survey.lt_The_response_to_quest_1]"
			continue
		    }
		}
	    }
	    
	    if { $abstract_data_type == "blob" } {
		set tmp_filename $response_to_question($question_id.tmpfile)
		set n_bytes [file size $tmp_filename]
		if { $n_bytes == 0 && $required_p == "t" } {
		    if {![info exist upload_file_exist($question_id)]} {
			ad_complain "[_ survey.lt_Your_file_is_zero-len]"
		    }
		}
	    }
	    
	}
	
	if { [llength $questions_with_missing_responses] > 0 } {
	    set missing ""
	    foreach skipped_question $questions_with_missing_responses {
		append missing "<li>$skipped_question"
	    }
	    ad_complain "You didn't resspond to all required questions. You skipped: $missing"
	    return 0
	} else {
	    return 1
	}
    }
    
} -properties {

    survey_name:onerow
}

ad_require_permission $survey_id write

set user_id [ad_conn user_id]
# anonymous user:
if {$user_id == 0} {
    set user_id [db_null]
}

get_survey_info -survey_id $survey_id
set type $survey_info(type)
set survey_id $survey_info(survey_id)
set survey_name $survey_info(name)


# Do the inserts.
# here we need to decide if it is an edit or multiple response, and create
# a new response, possibly linked to a previous response.

set response_id $response_id

set creation_ip [ad_conn peeraddr]
if {$initial_response_id==0} {
    set initial_response_id ""
}

foreach section_id $section_ids {
    db_transaction {
	
	if {[db_string get_response_count {}] == 0} {
	    db_exec_plsql create_response {}
	}
	
	set question_info_list [db_list_of_lists survey_question_info_list {}]

	# in case of editing a response, clear old responses
	db_dml clear_section_response ""
	
	foreach question $question_info_list { 
	    set question_id [lindex $question 0]
	    set question_text [lindex $question 1]
	    set abstract_data_type [lindex $question 2]
	    set presentation_type [lindex $question 3]
	    set required_p [lindex $question 4]
	    set predefined_question_id [lindex $question 5]

	    if {[info exists response_to_question($question_id)]} {
		if {$abstract_data_type=="date"} {
		    set response_value "$response_to_question($question_id.year)-$response_to_question($question_id.month)-$response_to_question($question_id.day)"
		} else {
		    set response_value [string trim $response_to_question($question_id)]
		}
	    } else {
		set response_value [db_null]
	    }
	    
	    # If this is a predefined question, handle it accordingly
	    if {$predefined_question_id != [db_null]} {
		db_1row predefined ""
		if {$action_type=="db"} {
		    # insert value into db
		    db_dml ignore "update $table_name set $column_name=:response_value where $key_name=:user_id"
		    if {![db_resultrows]} {
			db_dml insert "insert into $table_name ($key_name,$column_name) values (:user_id,_response_value)"
		    }
		} else {
		    if {$action_type=="tcl"} {
			# execute tcl code - need more specs on that - Kolja
		    }
		}
	    }

	    switch -- $abstract_data_type {
		"choice" {
		    if { $presentation_type == "checkbox" } {
			# Deal with multiple responses. 
			set checked_responses $response_to_question($question_id)
			foreach response_value $checked_responses {
			    if { [empty_string_p $response_value] } {
				set response_value [db_null]
			    }
			    
			    db_dml survey_question_response_checkbox_insert "insert into survey_question_responses (response_id, question_id, choice_id)
 values (:response_id, :question_id, :response_value)"
			}
		    }  else {
			if { [empty_string_p $response_value] || [empty_string_p [lindex $response_value 0]] } {
			    set response_value [db_null]
			}

			db_dml survey_question_response_choice_insert "insert into survey_question_responses (response_id, question_id, choice_id)
 values (:response_id, :question_id, :response_value)"
		    }
		}
		"shorttext" {
		    db_dml survey_question_choice_shorttext_insert "insert into survey_question_responses (response_id, question_id, varchar_answer)
 values (:response_id, :question_id, :response_value)"
		}
		"boolean" {
		    if { [empty_string_p $response_value] } {
			set response_value [db_null]
		    }
		    db_dml survey_question_response_boolean_insert "insert into survey_question_responses (response_id, question_id, boolean_answer)
 values (:response_id, :question_id, :response_value)"
		}
		"integer" -
		"number" {
		    if { [empty_string_p $response_value] } {
			set response_value [db_null]
		    } 
		    db_dml survey_question_response_integer_insert "insert into survey_question_responses (response_id, question_id, number_answer)
 values (:response_id, :question_id, :response_value)"
		}
		"text" {
		    if { [empty_string_p $response_value] } {
			set response_value [db_null]
		    }
		    
		    db_dml survey_question_response_text_insert {} -clobs [list $response_value]
	    }
	    "date" {
                if { [empty_string_p $response_value] } {
                    set response_value [db_null]
                }

		db_dml survey_question_response_date_insert {}
	    }   
            "blob" {

                if { ![empty_string_p $response_value] } {
                    # this stuff only makes sense to do if we know the file exists
                    # cannot use [template::data::transform::file because of the . in the var name
                    set filename     [ns_queryget response_to_question.$question_id]
                    set tmp_filename [ns_queryget response_to_question.$question_id.tmpfile]
                    set mime_type    [ns_queryget response_to_question.$question_id.content-type]
                    set file_extension [string tolower [file extension $response_value]]
                    # remove the first . from the file extension
                    regsub {\.} $file_extension "" file_extension

                    set n_bytes [file size $tmp_filename]

                    if { $n_bytes == 0 } {
                        error "[_ survey.lt_This_should_have_been]"
                    } else {
                         set unique_name "${filename}_${response_id}"
			if {[ad_conn user_id]=="0"} {
			    # anonymous answer
			    set creation_user [db_null]
			} else {
			    set creation_user [ad_conn user_id]
			}
                         if {[info exist upload_file_exist($question_id)]} {
                             set old_revision_id $upload_file_exist($question_id)
                             set old_item [db_string get_item_id ""]
                             set revision_id [cr_import_content -item_id $old_item -creation_user $creation_user -title $filename "" $tmp_filename $n_bytes $mime_type $unique_name ]
                             db_dml survey_question_response_file_attachment_update ""
                             db_dml update_item_description ""
                         } else {
                             set revision_id [cr_import_content -creation_user $creation_user -title $filename "" $tmp_filename $n_bytes $mime_type $unique_name ]
                             db_dml survey_question_response_file_attachment_insert ""
                         }
                         # we use cr_import_content now --DaveB
                         # this abstracts out for use the blob handling for oracle or postgresql
                         # we are linking the file item_id to the survey_question_response attachment_answer field now

	 	    }
                }
            }
	}
    }
}
}

# Look for next section
set section_ids [survey_next_sections -edit_p $edit_p $survey_id $section_id $response_id]
if {$section_ids!=""} {
    # found following section
    ad_returnredirect "respond.tcl?[export_vars -url {survey_id response_id edit_p section_ids:multiple return_url}]"
} else {
    # survey is finished
    db_dml finish_response {}
}

survey_do_notifications -response_id $response_id




if {[exists_and_not_null return_url]} {
    ad_returnredirect "$return_url?[export_vars -url survey_id]"
           ad_script_abort
} else {
     set context_bar [ad_context_bar "[_ survey.lt_Response_Submitted_fo]"]
     ad_return_template
}	
