ad_page_contract {

    Allow the user to modify a question.

    @param   section_id   survey this question belongs to
    @param   question_id question which text we're changing

    @author  cmceniry@arsdigita.com
    @author  nstrug@arsdigita.com
    @date    Jun 16, 2000
    @cvs-id  $Id: question-modify.tcl,v 1.7 2005/05/13 02:20:13 jeffd Exp $
} {

    question_id:integer
    section_id:integer
    {valid_responses ""}
    {presentation_options ""}
    {sort_order ""}
}

get_survey_info -section_id $section_id
get_section_info -section_id $section_id
set survey_name $survey_info(name)
set survey_id $survey_info(survey_id)
ad_require_permission $survey_id admin
set allow_question_deactivation [ad_parameter "allow_question_deactivation_p"]
set n_responses [db_string survey_number_responses {} ]
set block_p $section_info(block_section_p)

ad_form -name modify_question -form {
    question_id:key
}

if {$n_responses > 0} {
    if {$n_responses >1} {
	set isare "[_ survey.are]"
	set resp "[_ survey.responses]"
    } else {
	set isare "[_ survey.is]"
	set resp "[_ survey.response]"
    }
    ad_form -extend -name modify_question -form {
	{warning:text(inform) {label "[_ survey.Warning]"} {value "<span style=\"color: #f00;\">[_ survey.lt_There_isare_n_respons]"}}
    }
}

ad_form -extend -name modify_question -export {sort_order} -form {
    {question_number:text(inform) {label "[_ survey.Modify_Question__]"}}
    {survey_name:text(inform) {label "[_ survey.From_1]"} {value $survey_name}}
    {question_text:text(textarea) {label "[_ survey.Question]"} {html {rows 5 cols 70}}}
    {pretty_id:text(hidden)
	(value $question_id)
    }
}

if {$allow_question_deactivation == 1} {
    ad_form -extend -name modify_question -form {
        {active_p:text(radio)     {label "[_ survey.Active]"} {options {{Yes t} {No f}}}}
    }
} else {
    ad_form -extend -name modify_question -form {
        {active_p:text(hidden) {value t}}
    }
}

ad_form -extend -name modify_question -form {
    {required_p:text(radio)     {label "[_ survey.Required]"} {options {{Yes t} {No f}}}}
    {section_id:text(hidden) {value $section_id}}
    {survey_id:text(hidden) {value $survey_id}}
} 


db_1row presentation {}

if {($presentation_type=="checkbox" || $presentation_type=="select" || $presentation_type=="radio") && $abstract_data_type != "boolean" && $block_p=="f"} {
    ad_form -extend -name modify_question -form {
	{presentation_type:text(radio) {
	    options {
		{"[_ survey.Radio_buttons]" radio} 
		{"[_ survey.Checkboxes]" checkbox} 
		{"[_ survey.Select_box]" select} 
		{"[_ survey.Radio_buttons_text]" radio_text} 
		{"[_ survey.Checkboxes_text]" checkbox_text} 
		{"[_ survey.Select_box_text]" select_text}
	    }
	} 
	    {value $presentation_type} 
	    {label "[_ survey.Presentation]"}
	}
    }

    set response_list ""
    db_foreach survey_question_valid_responses "" {
	if {![empty_string_p $numeric_value]} {
	    append valid_responses "[join [list $label $numeric_value] ","]\n"
	} else {
	    append valid_responses "${label}\n"
	}
    }
    ad_form -extend -name modify_question -form {
        {valid_responses:text(textarea)
            {label "[_ survey.lt_For_Multiple_Choicebr]"}
            {html {rows 10 cols 50}}
            {value $valid_responses}}
    }
}

if {$presentation_type == "textarea" || $presentation_type == "textbox"} {
    ad_form -extend -name modify_question -form {
	{presentation_options:text(select) {options {{Small small} {Medium medium} {Large large}}} {value $presentation_options} {label "[string totitle $presentation_type] Size"}}
    }
}

if {$section_info(block_section_p)=="f"} {
    set sections [db_list_of_lists sections ""]
    ad_form -extend -name modify_question -form {
	{new_section:integer(select) {options $sections}  {label "[_ survey.Move_to_section]"} {value $section_id}}
    }
}

ad_form -extend -name modify_question -select_query_name {survey_question_details} -edit_data {

    db_transaction {
	if {[info exists new_section]} {
	    set anchor [db_string new_anchor ""]
	    set section_id $new_section
	}
	
	db_dml survey_question_update {}
	
	# add new responses is choice type question
	
	if {[info exists valid_responses] && $block_p=="f"} {
	    
	    set responses [split $valid_responses "\n"]
	    set count 0
	    set response_list ""
	    foreach response $responses {
		set numeric_value ""
		regexp {(.+),\s*(\d+)} $response discard response numeric_value
		set trimmed_response [string trim $response]
		set numeric_value [string trim $numeric_value]
		if { [empty_string_p $trimmed_response] } {
		    # skip empty lines
		    continue
		}
		
		lappend response_list [list "$trimmed_response" "$count" ${numeric_value}]
		incr count
	    }
	    
	    set choice_id_to_update_list [db_list get_choice_id {}]
	    set sort_order 0
	    foreach one_response $response_list {
		set choice_label [lindex $one_response 0]
		set sort_order [lindex $one_response 1]
		set numeric_value [lindex $one_response 2]
		set choice_id_to_update [lindex $choice_id_to_update_list $sort_order]
		if {[empty_string_p $choice_id_to_update]} {
		    set new_choice_id [db_nextval survey_choice_id_sequence]
		    db_dml insert_new_choice {}
		} else {
		    db_dml update_new_choice {}
		}
		incr sort_order
	    }

	    while {[llength $choice_id_to_update_list] >= $sort_order} {
		set choice_id_to_delete [lindex $choice_id_to_update_list $sort_order]
		db_dml delete_old_choice {}
		incr sort_order
	    }
	    
	}
    } on_error {

            db_release_unused_handles
            ad_return_error "Database Error" "<pre>$errmsg</pre>"
            ad_script_abort
 
    }
	
	
	
	
    ad_returnredirect "one?survey_id=$survey_id&#${sort_order}"
    ad_script_abort
}



set context [list [list "one?survey_id=$survey_id" $survey_info(name)] [_ survey.Modify_Question]]

ad_return_template
