ad_page_contract {

    Set up template and see variables for a report based on a given survey.
    WARNING: This page has a file storage proc in it that will need to be moved if this goes into the standard survey package.

    @param   survey_id   section we're adding a template to

    @author  Caroline@Meekshome.com
    @date    Feb 2004
    @cvs-id  $Id: report.tcl,v 1.3 2005/03/16 12:26:31 jeffd Exp $
} {
    survey_id:integer
}

ad_require_permission $survey_id admin

get_survey_info -survey_id $survey_id
if {![info exists survey_info(survey_id)]} {
    ad_return_complaint 1 "[_ survey.lt_Requested_survey_does]"
    ad_script_abort
}

set survey_name $survey_info(name)

set context [list [list "one?[export_url_vars survey_id]" $survey_info(name)] [_ survey.Respondents]]

# If there is already a report for this survey load it.

set template_filename "[acs_package_root_dir "survey"]/www/reports/${survey_id}.adp"

if {[file exists $template_filename]} {
    set templatesource [template::util::read_file $template_filename]
} else {
    set templatesource ""
}
 
#Our goal here is to make as easy a UI as possible. So if a question has a distinct pretty_id we can call the variable that. However our constraint is on pretty_id, section_id so we need to check for duplicates in multisection surveys.

set duplicate_pretty_ids [db_list duplicate_pretty_ids {select distinct(q1.pretty_id) 
       from survey_questions q1 right outer join survey_sections s on q1.section_id = s.section_id
where survey_id = :survey_id
and 1 < (select count(*) from survey_questions q2 right outer join survey_sections s on q2.section_id = s.section_id
     where survey_id = :survey_id
and q2.pretty_id = q1.pretty_id)}]

# get all the questions and make variables for them based on their pretty_ids
db_multirow -extend {short_id full_id possible_values} vars survey_q_and_a {      select s.name as name, question_id, question_text, presentation_type, sort_order, active_p, required_p, abstract_data_type,
     s.section_id as section_id, branch_p, 
     coalesce(q.pretty_id,(question_id::varchar)) as question_pretty_id, 
     coalesce(s.pretty_id,(s.section_id::varchar)) as section_pretty_id
     from survey_questions q right outer join survey_sections s
     on (q.section_id = s.section_id)
     where survey_id=:survey_id
     and abstract_data_type in ('text','shorttext','boolean','number','integer','choice')
     order by s.sort_key,s.section_id, sort_order} {
    #The goal of this section is to list for the user all the variables he has available when he makes his template and the possible answers.
	 set full_id [fs::remove_special_file_system_characters -string "$section_pretty_id-$question_pretty_id"]
	 if { [lsearch $question_pretty_id $duplicate_pretty_ids] == -1 } {
	     set short_id [fs::remove_special_file_system_characters -string "$question_pretty_id"]
	 }
	 switch $abstract_data_type {
	"choice" {
	    set answer_list [db_list answers {select label from survey_question_choices where question_id = :question_id}]
	    set possible_values "[join $answer_list "<br>"]"
	} 
	"text" -
	"shorttext" {
	    set possible_values "<em>text string</em>"
	}
	"boolean" {
	    set possible_values "<em>boolean (use &lt;if &#64;var&#64; true> or &lt;if &#64;var&#64; false&gt;)</em>"
	}
	default {
	    set possible_values "<em>$abstract_data_type</em>"
	}
    }

}


#We need to change this to something that versions the template. This is too dangerous.

ad_form -name edit_template -form {
    {survey_id:text(hidden) {value $survey_id}}
    {template_source:text(textarea) {label "[_ survey.Template_Code]"} {html {rows 20 cols 80}} {value $templatesource}}
    {apply:text(submit) {label "[_ survey.Apply]"} {value "apply"}}
} -on_submit {
    ns_log notice "CM: apply=$apply"
    if {$apply=="Apply"} {
	# write edited template into a file
	ns_log notice "new template is: $template_source"
	set file_handle [open $template_filename w]
	puts $file_handle $template_source
	close $file_handle	
    } 
    ad_returnredirect "report?survey_id=$survey_id"
    
}


set context_bar [ad_context_bar "Create a Report"]

ad_return_template

