ad_page_contract {

    Display a questionnaire for one survey.

    @param  survey_id   id of displayed survey
    @param  section_ids sections that should be displayed

    @author philg@mit.edu
    @author nstrug@arsdigita.com
    @date   28th September 2000
    @cvs-id $Id: respond.tcl,v 1.4 2005/03/16 12:26:31 jeffd Exp $

} {
    
    survey_id:integer,notnull
    {section_ids:integer,multiple 0}
    {response_id:integer 0}
    {return_url:optional "/pvt/home"}
    {edit_p "f"}
} -validate {
    survey_exists -requires {survey_id} {
	if ![db_0or1row survey_exists {}] {
	    ad_complain "[_ survey.lt_Survey_survey_id_does_1]"
	}
	#set user_id [auth::require_login]
	# Allow public surveys
	set user_id [ad_conn user_id]
	set count_resp [db_0or1row count_responses {}]
	# If we have a response in the system, and the survey is editable
	# set the response_id to the latest response unless response_id is
	# submited when calling the function
	if {$number_of_responses!=0 && $response_id==0} {
	    set response_id $max_response_id
	}
	
	get_survey_info -survey_id $survey_id
	#set single_section_p $survey_info(single_section_p)
	#    if {$section_id==0 && $single_section_p=="t"} {
	#        set section_id $survey_info(section_id)
	#    }
	set name $survey_info(name)
	set description $survey_info(description)
	set single_response_p $survey_info(single_response_p)
	set editable_p $survey_info(editable_p)
	set display_type $survey_info(display_type)
	if {($single_response_p=="t" && $editable_p=="f" && $number_of_responses>0) || ($single_response_p=="t" && $editable_p=="t" && $number_of_responses>0 && $response_id==0)} {
	    ad_complain "[_ survey.lt_You_have_already_comp]"
	} elseif {$response_id>0 && $editable_p=="f"} {
	    ad_complain "[_ survey.lt_This_survey_is_not_ed]"
	}
    }
} -properties {

    name:onerow
    section_id:onerow
    button_label:onerow
    questions:onerow
    description:onerow
    modification_allowed_p:onerow
    return_url:onerow
}

ad_require_permission $survey_id write

set context [list "$name"]
set button_label "[_ survey.Submit_response]"
if {$editable_p == "t"} {
    if {$response_id > 0} {
#	set button_label "Modify previous response"
	db_1row get_initial_response ""
    }
}

if {$section_ids == 0} {
    # If no section_id is given, start the survey
    # first check if there is already a response_id (edit)
    # if not check for an unfinished response
    # if yes, take the first unfinished section of that
    # if no, start a new
    if {$response_id==0} {
	db_1row unfinished_surveys ""
	if {$unfinished==0} {
	    # No unfinished, start a new response
	    set response_id [db_nextval acs_object_id_seq]
	}
	set section_ids [survey_next_sections $survey_id 0 $response_id]
    } else {
	set section_ids [survey_next_sections -edit_p t $survey_id 0 $response_id]
    }
}

if {[llength $section_ids]==0} {
    # No following section, so mark this response finished and start a new one.
    # Normally this should already be handled in process-response
    db_dml finish_survey ""
    set response_id [db_nextval acs_object_id_seq]
    set section_ids [survey_next_sections -edit_p t $survey_id 0 $response_id]
}

# DEDS: add this to track page numbers
survey_get_section_page_numbers -survey_id $survey_id -array page_num
set current_page_num $page_num([lindex $section_ids 0])
set total_page_num $page_num(total)
set page_description [db_string get_desc "
    select description
    from survey_sections
    where section_id = [lindex $section_ids 0]
" -default 0]
set total_questions [db_string get_question_count {
    select count(*)
    from survey_questions q, survey_sections s
    where q.section_id = s.section_id
          and s.survey_id = :survey_id
          and q.active_p = 't'
} -default 0]
set finished_questions [db_string get_finished_question_count "
    select count(*)
    from survey_questions q, survey_sections s
    where q.section_id = s.section_id
          and s.survey_id = :survey_id
          and q.active_p = 't'
          and s.section_id in (select s2.section_id
                               from survey_sections s2
                               where s2.sort_key < (select s3.sort_key from survey_sections s3 where section_id = '[lindex $section_ids 0]')
                                     and survey_id = :survey_id
                              )
" -default 0]

ad_form -name take_survey -action process-response -html {enctype multipart/form-data} -form {
    {survey_id:text(hidden) {value $survey_id}}
    {section_ids:text(hidden) {value $section_ids}}
    {response_id:text(hidden) {value $response_id}}
    {return_url:text(hidden) {value $return_url}}
}

foreach section_id $section_ids {
    survey_section_add_to_form take_survey $section_id $response_id
}

# status bar
set status_bar [survey_status_bar [lindex $section_ids 0]]

# select survey style
set style "survey/standard"
db_0or1row get_style ""

#This is a complete Hack. I have no idea how the section_template in survey_section_add_to_form is supposed to work.
#This only works if there is one section.

if {[llength $section_ids] == 1} {
        set template_filename "[template::get_resource_path]/forms/survey/sections/${section_id}.adp"
    if {[file exists $template_filename]} {
	set style "survey/sections/${section_id}"
    }
}


ad_return_template 
