# /packages/survey/www/admin/section-add-2.tcl
ad_page_contract {
    Inserts a new section into the database.

    @param survey_id               integer denoting which survey we're adding section to
    @param section_id              id of new section
    @param after                   integer determining position of this section
    @param name                    name of the new section
    @param description             description of the new section
    @param description_html_p      is the description in html?

    @author Jin Choi (jsc@arsdigita.com) February 9, 2000
    @author nstrug@arsdigita.com
    @cvs-id $Id: section-add-2.tcl,v 1.3 2005/01/13 13:54:40 jeffd Exp $
} {
    section_id:integer,notnull
    survey_id:integer,notnull
    after:integer,notnull
    description:html,notnull
    description_html_p:boolean,notnull
    name:notnull
    branch_p:boolean,notnull
    page_break_p:boolean,notnull
    {pretty_id "$section_id"}
} -validate {
    first_section_not_branch -requires {branch_p:notnull} {
	if {$branch_p=="t" && $after==0} {
	    ad_complain "[_ survey.lt_First_section_cannot_]"
	}
    }
}

set package_id [ad_conn package_id]
set user_id [ad_get_user_id]
ad_require_permission $package_id admin
get_survey_info -survey_id $survey_id

if {![info exists survey_info(survey_id)]} {
    ad_return_complaint 1 "[_ survey.lt_Requested_survey_does]"
    ad_script_abort
}

set exception_count 0
set exception_text ""

if { [empty_string_p $description] } {
    incr exception_count
    append exception_text "<li>[_ survey.lt_You_did_not_enter_a_d]\n"
}

if { $exception_count > 0 } {
    ad_return_complaint $exception_count $exception_text
    ad_script_abort
}

set already_inserted_p [db_string already_inserted_p {}]

if { $already_inserted_p } {
    ad_returnredirect "one?[export_vars survey_id]"
    ad_script_abort
}

db_transaction {
    # We're inserting between existing sections; move everybody down.
    set sort_key [expr { $after + 1 }]
    db_dml renumber_sort_keys {}
        
    db_exec_plsql create_section {}
    db_dml make_description ""
    if {$branch_p == "t"} {
	set branch_id [db_nextval survey_branch_id_sequence]
	db_dml add_branch ""
    }
    
} on_error {
    
    db_release_unused_handles
    ad_return_error "Database Error" "<pre>$errmsg</pre>"
    ad_script_abort
    
}


db_release_unused_handles
if {$branch_p == "t"} {
    ad_returnredirect "condition-add?section_id=$section_id"
    ad_script_abort
}

ad_returnredirect "one?survey_id=$survey_id&#${sort_key}"







