# /www/survsimp/admin/survey-create.tcl
ad_page_contract {

  Form for creating a survey.

  @param  name         title for new survey
  @param  description  description for new survey

  @author raj@alum.mit.edu
  @author nstrug@arsdigita.com
  @date   February 9, 2000
  @cvs-id $Id: survey-create.tcl,v 1.6 2005/05/05 13:44:03 jeffd Exp $

} {
    survey_id:optional
    {name ""}
    {desc:html ""}
    {variable_names ""}
    {type "general"}
    {category_id 0}
}

set package_id [ad_conn package_id]

# bounce the user if they don't have permission to admin surveys
ad_require_permission $package_id admin
set user_id [ad_conn user_id]

# use ad_form --DaveB
set display_type "list"


#ad_form -name create_survey -confirm_template survey-create-confirm -form
ad_form -name create_survey -form {
    survey_id:key
    {display_type:text(hidden)
        {value $display_type}}
    {name:text
        {label "[_ survey.Survey_Name_1]"}
        {html {size 55}}}
    {desc:richtext
        {label "[_ survey.Description_1]"}
        {html {rows 5 cols 40}}
        {richtext_size {height 80}}
        {value $desc}
        {help_text "[_ survey.Description_help]"}}
    {public_p:text(radio)
        {label "[_ survey.Public]"}
        {options {{Yes t} {No f}}}
        {help_text "[_ survey.Public_help]"}
        {value t}}
}

if {$category_id} {
    set category_name [category::get_name $category_id]
    ad_form -extend -name create_survey -form {
 	{category:text(inform) 
            {label "Associate with element type"}
            {value $category_name} }
    }
}

ad_form -extend -name create_survey -form {
    {category_id:text(hidden) {value $category_id}}
}

ad_form -extend -name create_survey -validate {
    {name {[string length $name] <= 4000}
        "[_ survey.lt_Survey_Name_must_be_4]"
    }
    {name {[string length $name] > 0}
        "[_ survey.lt_Please_enter_a_name_f]"
    }
    {desc {[string length $desc] <= 4000}
        "[_ survey.lt_Survey_Description_mu]"
    }
    {desc {[string length $desc] > 0}
        "[_ survey.lt_Please_enter_a_descri]"
    }
    {survey_id {[db_string count_surveys ""] < 1}
        "oops"
    }
} -new_data {
    set description [template::util::richtext::get_property contents $desc]
    set content_format [template::util::richtext::get_property format $desc]
    if {[string compare $content_format "plain"] == 0} {
	set description_html_p "f"
    } else {
	set description_html_p "t"
    }

    if {[parameter::get -package_id $package_id -parameter survey_enabled_default_p -default 0]} {
	set enabled_p "t"
    } else {
	set enabled_p "f"
    }
    db_transaction {
	set context_id $package_id
	db_exec_plsql create_survey ""

	if {$public_p=="t"} {
	    # survey is public: grant permission to take survey to The Public
	    set public_id [acs_magic_object "the_public"]
	    permission::grant -party_id $public_id -object_id $survey_id -privilege write
	}
	# survey type-specific inserts


	# create new section here. the questions go in the section
	# section_id is null to create a new section
	# we might want to specify a section_id later for
	# multiple section surveys
 	# DEDS: look for a category
 	if {$category_id} {
 	    category::map_object -remove_old -object_id $survey_id [list $category_id]
 	}

	set section_id ""
	set section_id [db_exec_plsql create_section ""]
    }
    ad_returnredirect "question-add?section_id=$section_id"
    ad_script_abort
}

# function to insert survey type-specific form html
set context [list [_ survey.Create_Survey]]
