ad_page_contract {

    Display a questionnaire for one survey.

    @param  survey_id   id of displayed survey
    @param  section_ids sections that should be displayed

    @author philg@mit.edu
    @author nstrug@arsdigita.com
    @date   28th September 2000
    @cvs-id $Id: respond.tcl,v 1.7 2005/03/23 22:23:40 jeffd Exp $

} {
    
    survey_id:integer,notnull
    {section_ids:integer,multiple 0}
    {response_id:integer 0}
    {return_url:optional index}
    {edit_p "f"}
} -validate {
    survey_exists -requires {survey_id} {
	if ![db_0or1row survey_exists {}] {
	    ad_complain "[_ survey.lt_Survey_survey_id_does_1]"
	}
	#set user_id [auth::require_login]
	# Allow public surveys
	set user_id [ad_conn user_id]
	set count_resp [db_0or1row count_responses {}]
	# If we have a response in the system, and the survey is editable
	# set the response_id to the latest response unless response_id is
	# submited when calling the function
	if {$number_of_responses!=0 && $response_id==0} {
	    set response_id $max_response_id
	}

	get_survey_info -survey_id $survey_id
	#set single_section_p $survey_info(single_section_p)
	#    if {$section_id==0 && $single_section_p=="t"} {
	#        set section_id $survey_info(section_id)
	#    }
	set name $survey_info(name_unquoted)
	set description $survey_info(description)
	set single_response_p $survey_info(single_response_p)
	set editable_p $survey_info(editable_p)
	set display_type $survey_info(display_type)
	if {($single_response_p=="t" && $editable_p=="f" && $number_of_responses>0) || ($single_response_p=="t" && $editable_p=="t" && $number_of_responses>0 && $response_id==0)} {
	    ad_complain "[_ survey.lt_You_have_already_comp]"
	} elseif {$response_id>0 && $editable_p=="f"} {
	    ad_complain "[_ survey.lt_This_survey_is_not_ed]"
	}
    }
} -properties {

    name:onerow
    section_id:onerow
    button_label:onerow
    questions:onerow
    description:onerow
    modification_allowed_p:onerow
    return_url:onerow
}

ad_require_permission $survey_id write

set context $name
set button_label "[_ survey.Submit_response]"
if {$editable_p == "t"} {
    if {$response_id > 0} {
#	set button_label "Modify previous response"
	db_1row get_initial_response ""
    }
}

if {$section_ids == 0} {
    # If no section_id is given, start the survey
    # first check if there is already a response_id (edit)
    # if not check for an unfinished response
    # if yes, take the first unfinished section of that
    # if no, start a new
    if {$response_id==0} {
	db_1row unfinished_surveys ""
	if {$unfinished==0} {
	    # No unfinished, start a new response
	    set response_id [db_nextval acs_object_id_seq]
	}
	set section_ids [survey_next_sections $survey_id 0 $response_id]
    } else {
	set section_ids [survey_next_sections -edit_p t $survey_id 0 $response_id]
    }
}

if {[llength $section_ids]==0} {
    # No following section, so mark this response finished and start a new one.
    # Normally this should already be handled in process-response
    db_dml finish_survey ""
    set response_id [db_nextval acs_object_id_seq]
    set section_ids [survey_next_sections -edit_p t $survey_id 0 $response_id]
}

ad_form -name take_survey -html {enctype multipart/form-data} -form {
    {survey_id:text(hidden) {value $survey_id}}
    {section_ids:text(hidden) {value $section_ids}}
    {response_id:text(hidden) {value $response_id}}
    {return_url:text(hidden) {value $return_url}}
}

foreach section_id $section_ids {
    survey::form::add_survey_section \
        -form take_survey \
        -section_id $section_id \
        -response_id $response_id \
        -edit_p $edit_p
}

ad_form \
    -extend  \
    -name take_survey \
    -after_submit {
        rp_internal_redirect "/packages/survey/www/process-response"
        ad_script_abort
    }

# status bar
set status_bar [survey_status_bar [lindex $section_ids 0]]

# select survey style
set style "survey/standard"
db_0or1row get_style ""

ad_return_template

